package cc.fascinated.model.user;

import cc.fascinated.model.user.history.History;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonProperty;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.Setter;
import lombok.ToString;
import lombok.extern.log4j.Log4j2;
import org.springframework.data.annotation.Id;
import org.springframework.data.mongodb.core.index.Indexed;
import org.springframework.data.mongodb.core.mapping.Document;

import java.util.UUID;

/**
 * @author Fascinated (fascinated7)
 */
@RequiredArgsConstructor
@Getter
@Setter
@Log4j2
@ToString
@Document("user")
public class User {
    /**
     * The ID of the user.
     */
    @Id
    @JsonIgnore
    private final UUID id;

    /**
     * The username of the user.
     * <p>
     * Usually their Steam name.
     * </p>
     */
    @Indexed
    private String username;

    /**
     * The ID of the users steam profile.
     */
    @Indexed
    @JsonProperty("id")
    private String steamId;

    /**
     * Whether the user has logged into the website.
     * <p>
     * This is used to determine if we should track their profiles or not.
     * If they haven't logged in, we don't want to track their profiles.
     * </p>
     */
    @JsonIgnore
    public boolean linkedAccount;

    /**
     * The user's ScoreSaber account.
     */
    public ScoreSaberAccount scoresaberAccount;

    /**
     * The user's statistic history.
     */
    public History history;

    /**
     * Gets the user's statistic history
     */
    public History getHistory() {
        if (this.history == null) {
            this.history = new History();
        }
        return this.history;
    }

    /**
     * Gets the user as a DTO.
     *
     * @return the user as a DTO
     */
    @JsonIgnore
    public UserDTO getAsDTO() {
        return new UserDTO(this.id, this.username, this.steamId, this.scoresaberAccount);
    }
}
